# Generated by Django 3.1.2 on 2020-10-22 09:59

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0050_scene_baselayer"),
    ]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="popup_config",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]
